package es.uca.xmleye.presentacion;

/*
 Copyright (C) 2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.io.File;
import java.util.List;
import java.util.ArrayList;
import javax.swing.ListModel;

import es.uca.xmleye.tipos.ExcepcionTipo;
import es.uca.xmleye.tipos.RepositorioTipos;
import es.uca.xmleye.tipos.Tipo;
import es.uca.xmleye.util.Par;

/**
 * Modelo de presentación para la ventana de tipos de documento aceptados.
 * 
 * @author Antonio García Domínguez
 * @version 1.0
 */
public class ModPresentacionVentanaTipos {
	private RepositorioTipos repositorio;

	// / Nombre de la variable de entorno a usar
	final static String CLAVE_ENTORNO_DIR_TIPOS = "XMLEYE_TYPES_DIR";

	// Ruta relativa al repositorio global
	final static String REPO_GLOBAL = "types";
	// Ruta relativa al repositorio local
	final static String REPO_LOCAL;

	// Identificador secuencial del elemento seleccionado
	private int seleccion;

	static {
		final String dirTipos = System.getenv(CLAVE_ENTORNO_DIR_TIPOS);
		REPO_LOCAL = dirTipos != null ? dirTipos : System
				.getProperty("user.dir");
	}

	public ModPresentacionVentanaTipos() throws ExcepcionTipo {
		cargar(new String[] { REPO_LOCAL, REPO_GLOBAL });
	}

	public ModPresentacionVentanaTipos(String[] fuentes) throws ExcepcionTipo {
		cargar(fuentes);
	}

	/** Devuelve el modelo a utilizar para construir la lista de tipos. */
	public ListModel getModeloLista() {
		return (ListModel) repositorio;
	}

	/** Devuelve el texto a mostrar en el campo Orden de Edición. */
	public String getTextoOrdenEdicion() {
		return seleccion != -1 ? ((Tipo) repositorio.getElementAt(seleccion))
				.getOrdenEdicion() : "";
	}

	/** Devuelve el texto a mostrar en el campo Orden de Importación. */
	public String getTextoOrdenImportacion() {
		return seleccion != -1 ? ((Tipo) repositorio.getElementAt(seleccion))
				.getOrdenImportacion() : "";
	}

	/** Devuelve el texto a mostrar en el campo Extensiones. */
	public String getTextoExtensiones() {
		final String[] exts = seleccion != -1 ? ((Tipo) repositorio
				.getElementAt(seleccion)).getExtensiones() : null;

		if (exts == null)
			return "";

		StringBuffer sBuf = new StringBuffer();
		if (exts.length > 0) {
			sBuf.append(exts[0]);
			for (int i = 1; i < exts.length; ++i) {
				sBuf.append(", ");
				sBuf.append(exts[i]);
			}
		}

		return sBuf.toString();
	}

	/** Devuelve el texto a mostrar en el campo Nombre. */
	public String getTextoNombre() {
		return seleccion != -1 ? ((Tipo) repositorio.getElementAt(seleccion))
				.getNombre() : "";
	}

	/**
	 * Devuelve la posición del elemento a seleccionar (-1 si no debe haber
	 * selección).
	 */
	public int getSeleccion() {
		return seleccion;
	}

	/**
	 * Devuelve una lista con la descripción y extensiones de cada tipo
	 * disponible.
	 * 
	 * Útil para un selector de ficheros, por ejemplo.
	 */
	public List<Par<String, String[]>> getElemsSelector() {
		List<Par<String, String[]>> lista = new ArrayList<Par<String, String[]>>();

		for (int i = 0; i < repositorio.getSize(); ++i) {
			final Tipo tipo = (Tipo) repositorio.getElementAt(i);
			lista.add(new Par<String, String[]>(tipo.getNombre(), tipo
					.getExtensiones()));
		}

		return lista;
	}

	/** Devuelve el tipo de documento al que corresponde el fichero. */
	public Tipo getTipoPorFich(File fich) {
		return repositorio.getTipoPorRuta(fich.getAbsolutePath());
	}

	/** El texto del campo Orden de Edición ha cambiado. */
	public void textoOrdenEdicionCambiado(String nuevoTexto) {
		if (seleccion == -1 || nuevoTexto.equals(getTextoOrdenEdicion()))
			return;
		((Tipo) repositorio.getElementAt(seleccion))
				.setOrdenEdicion(nuevoTexto);
	}

	/** El texto del campo Orden de Importación ha cambiado. */
	public void textoOrdenImportacionCambiado(String nuevoTexto) {
		if (seleccion == -1 || nuevoTexto.equals(getTextoOrdenImportacion()))
			return;
		((Tipo) repositorio.getElementAt(seleccion))
				.setOrdenImportacion(nuevoTexto);
	}

	/** El texto del campo Extensiones ha cambiado. */
	public void textoExtensionesCambiado(String nuevoTexto) {
		if (seleccion == -1 || nuevoTexto.equals(getTextoExtensiones()))
			return;
		final String[] exts = nuevoTexto.split("\\s*,\\s*");
		((Tipo) repositorio.getElementAt(seleccion)).setExtensiones(exts);
	}

	/** El texto del campo Nombre ha cambiado. */
	public void textoNombreCambiado(String nuevoTexto) {
		if (seleccion == -1 || nuevoTexto.equals(getTextoNombre()))
			return;
		((Tipo) repositorio.getElementAt(seleccion)).setNombre(nuevoTexto);
	}

	/** La selección ha cambiado. */
	public void seleccionCambiada(int nuevaPos) {
		seleccion = nuevaPos;
	}

	/** Se ha pulsado Aceptar. */
	public void cambiosConfirmados() throws ExcepcionTipo {
		repositorio.guardar();
	}

	/** Se ha pulsado Cancelar. */
	public void cambiosCancelados() throws ExcepcionTipo {
		repositorio.recargar();
	}

	/** Se ha pulsado Restaurar. */
	public void restauracionSolicitada() throws ExcepcionTipo {
		if (seleccion == -1)
			return;
		((Tipo) repositorio.getElementAt(seleccion)).restaurar();
	}

	/** METODOS PRIVADOS * */
	private void cargar(String[] fuentes) throws ExcepcionTipo {
		// El directorio del usuario es más prioritario que el global
		// del sistema
		repositorio = new RepositorioTipos(fuentes);
		seleccion = repositorio.getSize() > 0 ? 0 : -1;
	}
}