package es.uca.xmleye.tipos;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.io.File;
import java.io.FilenameFilter;
import javax.swing.AbstractListModel;

/**
 * Repositorio de tipos de documento disponibles.
 * 
 * Un repositorio de tipos de documento no es más que un directorio con una
 * serie de ficheros con extensión .type. Cada uno de ellos contiene la
 * definición de un tipo de documento que puede abrir XMLEye utilizando un
 * preprocesador previo.
 * 
 * Es posible definir un repositorio compuesto pasando varias rutas en vez de
 * una, combinando efectivamente todos los repositorios en uno:
 * <ul>
 * <li>En la carga: si dos repositorios contienen el mismo fichero, la entrada
 * del repositorio con mayor prioridad oculta a la otra.</li>
 * 
 * <li>Al guardar: toda entrada siempre se intentará guardar en el repositorio
 * con mayor prioridad, independientemente de que el tipo ya exista en otro
 * repositorio.</li>
 * </ul>
 * 
 * @author Antonio García Domínguez
 * @version 1.0
 */
public class RepositorioTipos extends AbstractListModel {
	/**
	 * Identificador para serialización
	 */
	private static final long serialVersionUID = -7757419290087998132L;
	private List<Tipo> entradas;
	private String[] rutas;

	/**
	 * Crea un nuevo repositorio compuesto.
	 * 
	 * @param rutas
	 *            Rutas de repositorios simples en orden descendiente de
	 *            prioridad.
	 * @exception ExcepcionTipo
	 *                Hubo un problema al cargar alguno de los repositorios.
	 */
	public RepositorioTipos(String[] rutas) throws ExcepcionTipo {
		cargar(rutas);
	}

	/**
	 * Crea un nuevo repositorio simple.
	 * 
	 * Método de conveniencia para <code>RepositorioTipos(String[])</code>.
	 * 
	 * @param ruta
	 *            Ruta relativa al directorio con las descripciones de los
	 *            tipos.
	 * @exception ExcepcionTipo
	 *                Hubo un problema al cargar los tipos.
	 */
	public RepositorioTipos(String ruta) throws ExcepcionTipo {
		cargar(new String[] { ruta });
	}

	/**
	 * Recarga los datos de las rutas empleadas.
	 * 
	 * Descarta todos los cambios que no se hayan guardado hasta el momento.
	 */
	public void recargar() throws ExcepcionTipo {
		cargar(rutas);
	}

	/** Guarda los cambios realizados en los tipos del repositorio. */
	public void guardar() throws ExcepcionTipo {
		for (Tipo t : entradas) {
			t.guardar();
		}
	}

	/**
	 * Devuelve el tipo de documento que corresponda.
	 * 
	 * @param ruta
	 *            Ruta al documento.
	 * @return Tipo de documento que le corresponde, o <code>null</code> si
	 *         ninguno coincide.
	 */
	public Tipo getTipoPorRuta(String ruta) {
		for (Tipo t : entradas) {
			for (String ext : t.getExtensiones()) {
				if (ruta.matches(".*\\." + ext))
					return t;
			}
		}

		return null;
	}

	/** IMPLEMENTACION DE ListModel * */
	public int getSize() {
		return entradas.size();
	}

	public Object getElementAt(int index) {
		return entradas.get(index);
	}

	/** METODOS PROTEGIDOS/PAQUETE * */
	void cargar(String[] rutas) throws ExcepcionTipo {
		this.rutas = rutas;
		Map<String, Tipo> mapaEntradas = new HashMap<String, Tipo>();

		for (String ruta : rutas) {
			final File dirBase = new File(ruta);
			for (String rutaTipo : dirBase.list(new FiltroTipo())) {
				final File fichTipo = new File(rutaTipo);
				final String clave = fichTipo.getName();
				if (mapaEntradas.containsKey(clave))
					continue;

				final File[] rutasTipo = new File[rutas.length];
				for (int i = 0; i < rutasTipo.length; ++i) {
					rutasTipo[i] = new File(rutas[i] + File.separator + clave);
				}

				mapaEntradas.put(clave, new Tipo(this, rutasTipo));
			}
		}

		entradas = new ArrayList<Tipo>(mapaEntradas.values());
	}

	List<Tipo> getTipos() {
		return entradas;
	}

	void nombreCambiado(Tipo cambiado) {
		int pos;
		for (pos = 0; pos < entradas.size() && cambiado != entradas.get(pos); ++pos)
			;

		if (pos < entradas.size()) {
			fireContentsChanged(this, pos, pos);
		}
	}

	/**
	 * Filtro por extensión .type
	 * 
	 * @author Antonio García Domínguez
	 * @version 1.0
	 */
	private static class FiltroTipo implements FilenameFilter {
		public boolean accept(File dir, String name) {
			return name.matches(".+\\.type");
		}
	}

}