package es.uca.xmleye.xslt;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.util.*;
import javax.xml.transform.*;
import java.io.*;
import javax.xml.transform.stream.*;

/**
 * <p>
 * URIResolver basado en una tabla hash. La correspondencia es varios-uno: un
 * misma hoja XSLT puede ser referenciada por varias URIs distintas.
 * </p>
 * 
 * <p>
 * Decora un resolvedor subordinado al que recurre cuando no es capaz de
 * resolver una URI.
 * </p>
 * 
 * <p>
 * <b>NOTA</b>: por lo pronto, no se usan las URI base en la resolución.
 * </p>
 * 
 * @author Antonio García Domínguez
 * @version 1.1
 */
public class ResolvedorURIHash implements URIResolver {
	private HashMap<String, File> resolveMap;

	// Resolvedor subordinado - se llama a éste cuando no hay coincidencia
	private URIResolver subordinado;

	/**
	 * Crea un nuevo resolvedor, que decora a <code>subordinado</code>.
	 * 
	 * @param subordinado
	 *            Resolvedor subordinado a éste.
	 */
	public ResolvedorURIHash(URIResolver subordinado) {
		this.resolveMap = new HashMap<String, File>();
		this.subordinado = subordinado;
	}

	public Source resolve(String href, String base) throws TransformerException {
		try {
			File f = resolveMap.get(href);
			if (f == null) {
				// No se halla en el diccionario
				return subordinado.resolve(href, base);
			}

			return new StreamSource(new BufferedInputStream(
					new FileInputStream(f)));
		} catch (FileNotFoundException fEx) {
			return null;
		}
	}

	/* ACCESO AL HASH */

	/**
	 * Cambia el valor asociado a una clave del diccionario de resolución.
	 * 
	 * @param clave
	 *            Clave del diccionario.
	 * @param f
	 *            Fichero con el documento XSLT a usar bajo esta clave.
	 */
	public void establecer(String clave, File f) {
		resolveMap.put(clave, f);
	}

	/**
	 * Accede al valor asociado a una clave del mapa de resolución.
	 * 
	 * @param clave
	 *            Clave del diccionario.
	 * @return f Fichero asociado a dicha clave, o <code>null</code> si no
	 *         está en el diccionario.
	 */
	public File obtener(String clave) {
		return resolveMap.get(clave);
	}

	/**
	 * Elimina el valor asociado a una clave del diccionario de resolución.
	 * 
	 * @param clave
	 *            Clave del diccionario.
	 */
	public void retirar(String clave) {
		resolveMap.remove(clave);
	}

}
